#include "ServerInstance.h"

#include <sstream>

using std::string;

template <class T>
std::string ToString(T num)
{
   std::ostringstream buf;

   buf << num;

   return buf.str();
}

static string GetLastErrorMessage(
   DWORD last_error,
   bool stripTrailingLineFeed = true)
{
   CHAR errmsg[512];

   if (!FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
      0,
      last_error,
      0,
      errmsg,
      511,
      NULL))
   {
      // if we fail, call ourself to find out why and return that error

      const DWORD thisError = ::GetLastError();

      if (thisError != last_error)
      {
         return GetLastErrorMessage(thisError, stripTrailingLineFeed);
      }
      else
      {
         // But don't get into an infinite loop...

         return "Failed to obtain error string: " + ToString(last_error);
      }
   }

   if (stripTrailingLineFeed)
   {
      const size_t length = strlen(errmsg);

      if (errmsg[length-1] == '\n')
      {
         errmsg[length-1] = 0;

         if (errmsg[length-2] == '\r')
         {
            errmsg[length-2] = 0;
         }
      }
   }

   return errmsg;
}

CServerInstance::CServerInstance(
   const char *pConfigValue,
   const WaspFunctions *pWaspFunctions)
   :  m_configValue(pConfigValue)
{
   memset(&m_waspFunctions, 0, sizeof(m_waspFunctions));

   memcpy(&m_waspFunctions, pWaspFunctions, sizeof(m_waspFunctions));

   m_waspFunctions.size = sizeof(m_waspFunctions);

   LogMessage("Instance created: " + ToString(this));
}

CServerInstance::~CServerInstance()
{
   LogMessage("Instance destroyed: " + ToString(this));
}

WaspConnectionUserData CServerInstance::OnConnectionEstablished(
   const WaspConnectionHandle connectionHandle,
   const sockaddr *pAddress,
   const WaspAddressLength addressLength)
{
   LogMessage(ToString(connectionHandle) + " - OnConnectionEstablished()");

   return 0;
}

void CServerInstance::OnReadCompleted(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const BYTE * const pDataIn,
   const WaspDataLength dataLength)
{
   LogMessage(ToString(connectionHandle) + " - OnReadCompleted()");

   (void)connectionUserData;

   const WaspError waspError = m_waspFunctions.writeToConnection(m_waspFunctions.id, connectionHandle, pDataIn, dataLength);

   if (WaspNoError != waspError)
   {
      const string message = ToString(connectionHandle) + " - writeToConnection failed - ";

      LogMessage(message + m_waspFunctions.getErrorMessageA(m_waspFunctions.id, waspError));
   }
}

void CServerInstance::OnConnectionClientClosed(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData)
{
   LogMessage(ToString(connectionHandle) + " - OnConnectionClientClosed()");
}

void CServerInstance::OnConnectionReset(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const WaspLastError lastError)
{
   LogMessage(ToString(connectionHandle) + " - OnConnectionReset() - " + ToString(lastError) + " - " + GetLastErrorMessage(lastError));
}

void CServerInstance::OnConnectionClosed(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData,
   const WaspConnectionClosureReason reason)
{
   LogMessage(ToString(connectionHandle) + " - OnConnectionClosed() - " + ToString(reason) + " - " + m_waspFunctions.getClosureReasonA(m_waspFunctions.id, reason));
}

void CServerInstance::OnConnectionComplete(
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData connectionUserData)
{
   LogMessage(ToString(connectionHandle) + " - OnConnectionComplete()");
}

void CServerInstance::OnShutdownInitiated()
{
   LogMessage("OnShutdownInitiated()");
}

void CServerInstance::LogMessage(
   const string &message)
{
   const string completeMessage = m_configValue + ": " + message;

   m_waspFunctions.logMessageA(m_waspFunctions.id, 0, completeMessage.c_str());
}
